use common::PORT_NUM;
use std::error::Error;
use std::process::{Command, Stdio};
use std::net::{ToSocketAddrs, Ipv4Addr, SocketAddr, SocketAddrV4, TcpStream};
use std::io::{Read, Write};
use std::io::Result as IoResult;
use std::time::Duration;

/// Discovery
/// Module responsible for discovering new instances.
/// Detects the device's local IP, then scans through the last octet, sending heartbeat messages
/// And adding those to the application-wide table of servers.

pub fn get_local_ip() -> SocketAddrV4 {

    let mut cmd_if = Command::new("ifconfig")
        .arg("eth0")
        .stdout(Stdio::piped())
        .spawn()
        .unwrap_or_else(|e| {
            panic!("failed to execute ifconfig: {}", e)
        });

    let mut stdout = cmd_if.stdout.expect("No stdout for ifconfig");
    let mut buf: Vec<u8> = Vec::new();
    stdout.read_to_end(&mut buf).unwrap();
    let ifc_output = String::from_utf8_lossy(&buf);
    println!("buf contents: {}", &ifc_output);

    let rind = ifc_output
                .find("inet addr:")
                .expect("No inet addr found for this interface.");
    let rslice = ifc_output.split_at(rind + 10).1; // 10 is length of "inet addr:"
    let mut addr_iter = rslice.split_whitespace();

    let mut addr = addr_iter
                .next()
                .expect("No more matches?")
                .to_string();

    let addr_tup = (&*addr, PORT_NUM);
    let addr_final = addr_tup.to_socket_addrs()
                        .expect("No iterator for socket addrs")
                        .next()
                        .expect("No matching socket_addr");

    //addr_final

    match addr_final {
        SocketAddr::V4(v4)  =>  v4,
        _                   => panic!("Nope.")
    }
}

///
pub fn scan_and_ping(addr: SocketAddrV4 ) {
    let ip = addr.ip();
    let ip_oct = ip.octets();

    // For now, we're scanning across two octets, because that's how my DigitalOcean droplets were configured.
    for i in 0..255 as u8 {
        for j in 0..255 as u8 {
            let scan_ip = SocketAddrV4::new(Ipv4Addr::new(ip_oct[0], ip_oct[1], i, j), PORT_NUM);
            if *scan_ip.ip() != *ip {
                println!("Scanning: {:?}", scan_ip);
                let res = attempt_heartbeat(&scan_ip);
                match res {
                    Ok(addr)    =>  println!("Heartbeat at addr: {:?}", addr),
                    _           =>  println!("No match at {:?}", scan_ip)
                }
            } else {
                println!("Skipping self.");
            }
        }
    }
}

///
fn attempt_heartbeat(addr: &SocketAddrV4) -> Result<SocketAddrV4, &str> {
    let stream_res = TcpStream::connect(addr);
    if stream_res.is_err() {
        Err("Connection refused")
    } else {
        // Safe to unwrap here
        let mut stream = stream_res.unwrap();
        let mut buf: [u8; 32] = [0; 32];
        stream.set_write_timeout(Some(Duration::new(3, 0)));
        stream.set_read_timeout(Some(Duration::new(3, 0)));
        let res = stream.write(b"hb");
        match res {
            Ok(_)   =>  Ok(addr.to_owned()),
            _       =>  Err("Connection failed")
        }
    }
}
